@@ -109,13 +109,9 @@ panel.barchart <-
              lwd = if (is.null(groups)) plot.polygon$lwd else superpose.polygon$lwd,
              ...)
 {
-
-    ## This is to make sure `levels' are calculated based on the whole
-    ## groups vector and not just the values represented in this
-    ## particular panel (which might make the key inconsistent and/or
-    ## cause other problems)
-
-    if (!is.null(groups) && !is.factor(groups)) groups <- factor(groups)
+    plot.polygon <- trellis.par.get("plot.polygon")
+    superpose.polygon <- trellis.par.get("superpose.polygon")
+    reference.line <- trellis.par.get("reference.line")
 
     ## this function doesn't have a subscripts argument (which would
     ## make barchart always pass the subscripts to the trellis object,
@@ -133,12 +129,21 @@ panel.barchart <-
     x <- as.numeric(x[keep])
     y <- as.numeric(y[keep])
 
-    plot.polygon <- trellis.par.get("plot.polygon")
-    superpose.polygon <- trellis.par.get("superpose.polygon")
-    reference.line <- trellis.par.get("reference.line")
+    if (!is.null(groups))
+    {
+        groupSub <- function(groups, subscripts, ...)
+            groups[subscripts[keep]]
+
+        ## This is to make sure `levels' are calculated based on the
+        ## whole groups vector and not just the values represented in
+        ## this particular panel (which might make the key
+        ## inconsistent and/or cause other problems)
+
+        if (!is.factor(groups)) groups <- factor(groups)
+        nvals <- nlevels(groups)
+        groups <- as.numeric(groupSub(groups, ...))
+    }
 
-    groupSub <- function(groups, subscripts, ...)
-        groups[subscripts[keep]]
 
     if (horizontal)
     {
@@ -172,11 +177,12 @@ panel.barchart <-
         else if (stack)
         {
             if (!is.null(origin) && origin != 0)
-                warning("origin forced to 0 for stacked bars")
-
-            groups <- as.numeric(groupSub(groups, ...))
-            vals <- sort(unique(groups))
-            nvals <- length(vals)
+                warning("'origin' forced to 0 for stacked bars")
+ 
+##             vals <- seq_len(nlevels(groups))
+##             groups <- as.numeric(groupSub(groups, ...))
+##             ## vals <- sort(unique(groups))
+##             nvals <- length(vals)
 
             col <- rep(col, length = nvals)
             border <- rep(border, length = nvals)
@@ -231,9 +237,10 @@ panel.barchart <-
                 origin <- current.panel.limits()$xlim[1]
                 reference <- FALSE
             }
-            groups <- as.numeric(groupSub(groups, ...))
-            vals <- sort(unique(groups))
-            nvals <- length(vals)
+##             vals <- seq_len(nlevels(groups))
+##             groups <- as.numeric(groupSub(groups, ...))
+##             ## vals <- sort(unique(groups))
+##             nvals <- length(vals)
 
             col <- rep(col, length = nvals)
             border <- rep(border, length = nvals)
@@ -294,11 +301,12 @@ panel.barchart <-
         {
 
             if (!is.null(origin) && origin != 0)
-                warning("origin forced to 0 for stacked bars")
+                warning("'origin' forced to 0 for stacked bars")
 
-            groups <- as.numeric(groupSub(groups, ...))
-            vals <- sort(unique(groups))
-            nvals <- length(vals)
+##             vals <- seq_len(nlevels(groups))
+##             groups <- as.numeric(groupSub(groups, ...))
+##             ## vals <- sort(unique(groups))
+##             nvals <- length(vals)
 
             col <- rep(col, length = nvals)
             border <- rep(border, length = nvals)
@@ -350,9 +358,10 @@ panel.barchart <-
                 origin <- current.panel.limits()$ylim[1]
                 reference = FALSE
             }
-            groups <- as.numeric(groupSub(groups, ...))
-            vals <- sort(unique(groups))
-            nvals <- length(vals)
+##             vals <- seq_len(nlevels(groups))
+##             groups <- as.numeric(groupSub(groups, ...))
+##             ## vals <- sort(unique(groups))
+##             nvals <- length(vals)
 
             col <- rep(col, length = nvals)
             border <- rep(border, length = nvals)
@@ -434,18 +443,12 @@ panel.stripplot <-
              factor = 0.5, amount = NULL,
              horizontal = TRUE, groups = NULL, ...)
 {
-    if (all(is.na(x) | is.na(y))) return()
-    x <- as.numeric(x)
-    y <- as.numeric(y)
-    if (jitter.data)
-    {
-        if (horizontal)
-            y[] <- jitter(y, factor = factor, amount = amount)
-        else
-            x[] <- jitter(x, factor = factor, amount = amount)
-    }
+    if (!any(is.finite(x) & is.finite(y))) return()
     panel.xyplot(x = x,
                  y = y,
+                 jitter.x = jitter.data && !horizontal,
+                 jitter.y = jitter.data &&  horizontal,
+                 factor = factor, amount = amount,
                  groups = groups,
                  horizontal = horizontal, ...)
 }
@@ -780,7 +783,7 @@ dotplot <- function(x, data, ...) UseMethod("dotplot")
 ## (e.g. dotplot(x, groups = a):
 
 ##     if (!missing(data))
-##         warning("explicit data specification ignored")
+##         warning("explicit 'data' specification ignored")
 ##     dotplot(~x, data = list(x = formula),
 ##             xlab = xlab,
 ##             ...)
@@ -789,13 +792,14 @@ dotplot <- function(x, data, ...) UseMethod("dotplot")
 dotplot.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- ccall <- match.call()
+    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(dotplot)
+    ccall <- match.call()
     if (!is.null(ccall$data)) 
-        warning("explicit data specification ignored")
+        warning("explicit 'data' specification ignored")
     ccall$data <- list(x = x)
     ccall$xlab <- xlab
     ccall$x <- ~x
-    ccall[[1]] <- as.name("dotplot")
+    ccall[[1]] <- quote(lattice::dotplot)
     ans <- eval.parent(ccall)
     ans$call <- ocall
     ans
@@ -803,15 +807,10 @@ dotplot.numeric <-
 
 
 
-
-
 dotplot.table <-
     function(x, data = NULL, groups = TRUE, ...)
 {
-    formula <- x
-    ocall <- match.call()
-    ## formula <- eval(substitute(~foo, list(foo = substitute(formula))))
-    data <- as.data.frame(formula)
+    data <- as.data.frame(x)
     nms <- names(data)
     freq <- which(nms == "Freq")
     nms <- nms[-freq]
@@ -830,11 +829,7 @@ dotplot.table <-
         rest <- paste(nms, collapse = "+")
         form <- paste(form, rest, sep = "|")
     }
-    ans <-
-        dotplot(as.formula(form), data, groups = eval(groups),
-                 ...)
-    ans$call <- ocall
-    ans
+    dotplot(as.formula(form), data, groups = eval(groups), ...)
 }
 
 
@@ -846,13 +841,14 @@ dotplot.matrix <- function(x, data = NULL, ...) dotplot(as.table(x), data, ...)
 dotplot.formula <-
     function(x,
              data = NULL,
-             panel = "panel.dotplot",
+             panel = lattice.getOption("panel.dotplot"),
              ...)
 {
-    ocall <- ccall <- match.call()
+    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(dotplot)
+    ccall <- match.call()
     ccall$data <- data
     ccall$panel <- panel
-    ccall[[1]] <- as.name("bwplot")
+    ccall[[1]] <- quote(lattice::bwplot)
     ans <- eval.parent(ccall)
     ans$call <- ocall
     ans
@@ -865,13 +861,14 @@ barchart <- function(x, data, ...) UseMethod("barchart")
 barchart.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- ccall <- match.call()
+    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(barchart)
+    ccall <- match.call()
     if (!is.null(ccall$data)) 
-        warning("explicit data specification ignored")
+        warning("explicit 'data' specification ignored")
     ccall$data <- list(x = x)
     ccall$xlab <- xlab
     ccall$x <- ~x
-    ccall[[1]] <- as.name("barchart")
+    ccall[[1]] <- quote(lattice::barchart)
     ans <- eval.parent(ccall)
     ans$call <- ocall
     ans
@@ -882,17 +879,15 @@ barchart.numeric <-
 
 barchart.table <-
     function(x, data = NULL, groups = TRUE,
-             origin = 0, stack = TRUE, ...)
+             origin = 0, stack = TRUE, ..., horizontal = TRUE)
 {
-    formula <- x
-    ocall <- match.call()
-    if (!is.null(data)) warning("explicit data specification ignored")
-    ## formula <- eval(substitute(~foo, list(foo = substitute(formula))))
-    data <- as.data.frame(formula)
+    if (!is.null(data)) warning("explicit 'data' specification ignored")
+    data <- as.data.frame(x)
     nms <- names(data)
     freq <- which(nms == "Freq")
     nms <- nms[-freq]
-    form <- paste(nms[1], "Freq", sep = "~")
+    form <- ## WAS paste(nms[1], "Freq", sep = "~")
+        sprintf(if (horizontal) "%s ~ Freq" else "Freq ~ %s", nms[1])
     nms <- nms[-1]
     len <- length(nms)
     if (is.logical(groups) && groups && len > 0)
@@ -907,14 +902,11 @@ barchart.table <-
         rest <- paste(nms, collapse = "+")
         form <- paste(form, rest, sep = "|")
     }
-    ans <-
-        barchart(as.formula(form), data,
-                 groups = eval(groups),
-                 ##groups = groups,
-                 origin = origin, stack = stack, 
-                 ...)
-    ans$call <- ocall
-    ans
+    barchart(as.formula(form), data,
+             groups = eval(groups),
+             ##groups = groups,
+             origin = origin, stack = stack, 
+             ...)
 }
 
 barchart.default <- function(x, data = NULL, ...) barchart(table(x), data, ...)
@@ -925,15 +917,16 @@ barchart.matrix <- function(x, data = NULL, ...) barchart(as.table(x), data, ...
 barchart.formula <-
     function(x,
              data = NULL,
-             panel = "panel.barchart",
+             panel = lattice.getOption("panel.barchart"),
              box.ratio = 2, 
              ...)
 {
-    ocall <- ccall <- match.call()
+    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(barchart)
+    ccall <- match.call()
     ccall$data <- data
     ccall$panel <- panel
     ccall$box.ratio <- box.ratio
-    ccall[[1]] <- as.name("bwplot")
+    ccall[[1]] <- quote(lattice::bwplot)
     ans <- eval.parent(ccall)
     ans$call <- ocall
     ans
@@ -946,16 +939,14 @@ stripplot <- function(x, data, ...)  UseMethod("stripplot")
 stripplot.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- ccall <- match.call()
+    ccall <- match.call()
     if (!is.null(ccall$data)) 
-        warning("explicit data specification ignored")
+        warning("explicit 'data' specification ignored")
     ccall$data <- list(x = x)
     ccall$xlab <- xlab
     ccall$x <- ~x
-    ccall[[1]] <- as.name("stripplot")
-    ans <- eval.parent(ccall)
-    ans$call <- ocall
-    ans
+    ccall[[1]] <- quote(lattice::stripplot)
+    eval.parent(ccall)
 }
 
 
@@ -963,13 +954,14 @@ stripplot.numeric <-
 stripplot.formula <-
     function(x,
              data = NULL,
-             panel = "panel.stripplot",
+             panel = lattice.getOption("panel.stripplot"),
              ...)
 {
-    ocall <- ccall <- match.call()
+    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(stripplot)
+    ccall <- match.call()
     ccall$data <- data
     ccall$panel <- panel
-    ccall[[1]] <- as.name("bwplot")
+    ccall[[1]] <- quote(lattice::bwplot)
     ans <- eval.parent(ccall)
     ans$call <- ocall
     ans
@@ -987,16 +979,14 @@ bwplot <- function(x, data, ...) UseMethod("bwplot")
 bwplot.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- ccall <- match.call()
+    ccall <- match.call()
     if (!is.null(ccall$data)) 
-        warning("explicit data specification ignored")
+        warning("explicit 'data' specification ignored")
     ccall$data <- list(x = x)
     ccall$xlab <- xlab
     ccall$x <- ~x
-    ccall[[1]] <- as.name("bwplot")
-    ans <- eval.parent(ccall)
-    ans$call <- ocall
-    ans
+    ccall[[1]] <- quote(lattice::bwplot)
+    eval.parent(ccall)
 }
 
 
@@ -1009,7 +999,7 @@ bwplot.formula <-
              outer = FALSE,
              auto.key = FALSE,
              aspect = "fill",
-             panel = "panel.bwplot",
+             panel = lattice.getOption("panel.bwplot"),
              prepanel = NULL,
              scales = list(),
              strip = TRUE,
@@ -1022,6 +1012,7 @@ bwplot.formula <-
              horizontal = NULL,
              drop.unused.levels = lattice.getOption("drop.unused.levels"),
              ...,
+             lattice.options = NULL,
              default.scales =
              if (horizontal) list(y = list(tck = 0, alternating = FALSE, rot = 0))
              else list(x = list(tck = 0, alternating = FALSE)),
@@ -1032,6 +1023,11 @@ bwplot.formula <-
     dots <- list(...)
     groups <- eval(substitute(groups), data, environment(formula))
     subset <- eval(substitute(subset), data, environment(formula))
+    if (!is.null(lattice.options))
+    {
+        oopt <- lattice.options(lattice.options)
+        on.exit(lattice.options(oopt), add = TRUE)
+    }
 
     ## step 0: hack to get appropriate legend with auto.key = TRUE in
     ## barchart (default panel only).  The usual default in bwplot is
@@ -1081,10 +1077,10 @@ bwplot.formula <-
     }
     if (horizontal)
     {
-        if (!(is.numeric(x)))
-        {
-            warning("x should be numeric")
-        }
+##         if (!(is.numeric(x)))
+##         {
+##             warning("x should be numeric")
+##         }
         y <- as.factorOrShingle(y)
         is.f.y <- is.factor(y)  # used throughout the rest of the code
         num.l.y <- nlevels(y)
@@ -1093,10 +1089,10 @@ bwplot.formula <-
     }
     else
     {
-        if (!(is.numeric(y)))
-        {
-            warning("y should be numeric")
-        }
+##         if (!(is.numeric(y)))
+##         {
+##             warning("y should be numeric")
+##         }
         x <- as.factorOrShingle(x)
         is.f.x <- is.factor(x)  # used throughout the rest of the code
         num.l.x <- nlevels(x)
@@ -1117,11 +1113,12 @@ bwplot.formula <-
                        xlab = xlab,
                        ylab = ylab,
                        xlab.default = form$right.name,
-                       ylab.default = form$left.name), dots))
+                       ylab.default = form$left.name,
+                       lattice.options = lattice.options), dots))
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- match.call()
+    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(bwplot)
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
