@@ -1,3 +1,19 @@
+#  File src/library/base/R/frametools.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 subset.data.frame <- function (x, subset, select, drop = FALSE, ...)
 {
     if(missing(subset))
@@ -40,7 +56,7 @@ subset.matrix <- function(x, subset, select, drop = FALSE, ...)
 }
 
 ### Notice use of "illegal" variable name for the first argument
-### This used to be "x", but then you couldn't create a variable 
+### This used to be "x", but then you couldn't create a variable
 ### called "x"...
 
 transform.data.frame <- function (`_data`, ...)
@@ -65,51 +81,3 @@ transform <- function(`_data`,...) UseMethod("transform")
 ## that. This is probably marginally useful at best. --pd
 transform.default <- function(`_data`,...)
     transform.data.frame(data.frame(`_data`),...)
-
-stack.data.frame <- function(x, select, ...)
-{
-    if (!missing(select)) {
-	nl <- as.list(1:ncol(x))
-	names(nl) <- names(x)
-	vars <- eval(substitute(select),nl, parent.frame())
-        x <- x[, vars, drop=FALSE]
-    }
-    x <- x[, unlist(lapply(x, is.vector)), drop = FALSE]
-    data.frame(values = unlist(unname(x)),
-               ind = factor(rep.int(names(x), lapply(x, length))))
-}
-
-stack <- function(x, ...) UseMethod("stack")
-
-stack.default <- function(x, ...)
-{
-    x <- as.list(x)
-    x <- x[unlist(lapply(x, is.vector))]
-    data.frame(values = unlist(unname(x)),
-               ind = factor(rep.int(names(x), lapply(x, length))))
-}
-
-unstack.data.frame <- function(x, form = formula(x), ...)
-{
-    form <- as.formula(form)
-    if (length(form) < 3)
-        stop("'form' must be a two-sided formula")
-    res <- c(tapply(eval(form[[2]], x), eval(form[[3]], x), as.vector))
-    if (length(res) < 2 || any(diff(unlist(lapply(res, length))) != 0))
-        return(res)
-    data.frame(res)
-}
-
-unstack <- function(x, ...) UseMethod("unstack")
-
-unstack.default <- function(x, form, ...)
-{
-    x <- as.list(x)
-    form <- as.formula(form)
-    if (length(form) < 3)
-        stop("'form' must be a two-sided formula")
-    res <- c(tapply(eval(form[[2]], x), eval(form[[3]], x), as.vector))
-    if (length(res) < 2 || any(diff(unlist(lapply(res, length))) != 0))
-        return(res)
-    data.frame(res)
-}
