@@ -63,13 +63,17 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                         call. = FALSE, domain = NA)
             if(.Platform$OS.type == "unix") {
                 platform <- built$Platform
-		if(length(grep("\\w", platform)) &&
+                r_arch <- .Platform$r_arch
+                ## allow mismatches if r_arch is in use, e.g.
+                ## i386-gnu-linux vs x86-gnu-linux depending on
+                ## build system.
+		if(!nzchar(r_arch) && length(grep("\\w", platform)) &&
                    !testPlatformEquivalence(platform, R.version$platform))
                     stop(gettextf("package '%s' was built for %s",
                                   pkgname, platform),
                          call. = FALSE, domain = NA)
                 ## if using r_arch subdirs, check for presence
-                if(nzchar(r_arch <- .Platform$r_arch)
+                if(nzchar(r_arch)
                    && file.exists(file.path(pkgpath, "libs"))
                    && !file.exists(file.path(pkgpath, "libs", r_arch)))
                     stop(gettextf("package '%s' is not installed for 'arch=%s'",
