@@ -1,3 +1,19 @@
+#  File src/library/base/R/source.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 source <-
 function(file, local = FALSE, echo = verbose, print.eval = echo,
 	 verbose = getOption("verbose"),
@@ -108,7 +124,7 @@ function(file, local = FALSE, echo = verbose, print.eval = echo,
             	dep <- paste(prompt.echo,
             		     gsub("\n", paste("\n", continue.echo, sep=""), dep),
             		     sep="")
-		nd <- nchar(dep, "chars") - 1
+		nd <- nchar(dep, "c") - 1
 	    } else {
 	    	if (i == 1) lastshown <- min(skip.echo, srcref[3]-1)
 	    	dep <- getSrcLines(srcfile, lastshown+1, srcref[3])
@@ -120,7 +136,7 @@ function(file, local = FALSE, echo = verbose, print.eval = echo,
 	    	}
 	    	dep <- paste(rep.int(c(prompt.echo, continue.echo), c(leading, length(dep)-leading)),
 	    		     dep, sep="", collapse="\n")
-	    	nd <- nchar(dep, "chars")
+	    	nd <- nchar(dep, "c")
 	    }
 	    if (nd) {
 		do.trunc <- nd > max.deparse.length
