@@ -290,6 +290,18 @@ static void SaveAsWin(NewDevDesc *dd, const char *display,
         PrivateCopyDevice(dd, ndd, display);
 }
 
+static void init_PS_PDF(void)
+{
+    SEXP call, initS, grNS=R_FindNamespace(mkString("grDevices"));
+
+    initS = findVarInFrame3(grNS, install("initPSandPDFfonts"), TRUE);
+    if(initS == R_UnboundValue)
+	error("missing initPSandPDFfonts() in grDevices namespace: this should not happen");
+    PROTECT(call = lang1(initS));
+    eval(call, R_GlobalEnv);
+    UNPROTECT(1);
+}
+
 
 static void SaveAsPostscript(NewDevDesc *dd, const char *fn)
 {
@@ -343,6 +355,8 @@ static void SaveAsPostscript(NewDevDesc *dd, const char *fn)
 	    }
 	}
     }
+    /* need to initialize PS/PDF font database */
+    init_PS_PDF();
     if (PSDeviceDriver(ndd, fn, paper, family, afmpaths, encoding,
                        bg, fg,
 		       fromDeviceWidth(toDeviceWidth(1.0, GE_NDC, gdd),
@@ -402,6 +416,7 @@ static void SaveAsPDF(NewDevDesc *dd, const char *fn)
 	    }
 	}
     }
+    init_PS_PDF();
     if (PDFDeviceDriver(ndd, fn, "special", family, afmpaths, encoding,
                         bg, fg,
 			fromDeviceWidth(toDeviceWidth(1.0, GE_NDC, gdd),
@@ -2076,10 +2091,10 @@ static void GA_NewPage(R_GE_gcontext *gc,
     if (xd->kind != SCREEN) {
 	xd->needsave = TRUE;
 	xd->clip = getrect(xd->gawin);
-	if(R_OPAQUE(xd->bg) || xd->kind == PNG ||
-	   xd->kind == BMP || xd->kind == JPEG )
-	    DRAW(gfillrect(_d, R_OPAQUE(xd->bg) ? xd->bgcolor : PNG_TRANS,
-			   xd->clip));
+	if(R_OPAQUE(xd->bg) || xd->kind == BMP || xd->kind == JPEG) {
+	    DRAW(gfillrect(_d, xd->bgcolor, xd->clip));
+	} else if(xd->kind == PNG)
+	    DRAW(gfillrect(_d, PNG_TRANS, xd->clip));
 	if(xd->kind == PNG) xd->pngtrans = ggetpixel(xd->gawin, pt(0,0));
     } else {
 	xd->clip = getregion(xd);
