@@ -1,4 +1,5 @@
 #include "grid.h"
+#include <string.h>
 
 /* Get the list element named str, or return NULL. 
  * Copied from the Writing R Extensions manual (which copied it from nls) 
@@ -177,7 +178,7 @@ void textRect(double x, double y, SEXP text, int i,
 	h = fromDeviceHeight(GEExpressionHeight(expr, gc, dd),
 			     GE_INCHES, dd);
     } else {
-	char* string = translateChar(STRING_ELT(text, i % LENGTH(text)));
+	const char* string = translateChar(STRING_ELT(text, i % LENGTH(text)));
 	w = fromDeviceWidth(GEStrWidth(string, gc, dd),
 			    GE_INCHES, dd);
 	h = fromDeviceHeight(GEStrHeight(string, gc, dd),
