@@ -1,3 +1,19 @@
+#  File src/library/stats/R/HoltWinters.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 HoltWinters <-
 function (x,
 
@@ -181,7 +197,7 @@ function (x,
                        trend  = if (beta > 0) final.fit$trend[-len-1],
                        season = if (gamma > 0) final.fit$seasonal[1:len]),
                  start = start(lag(x, k = 1 - start.time)),
-                 freq  = frequency(x)
+                 frequency  = frequency(x)
                  )
     if (beta > 0) fitted[,1] <- fitted[,1] + fitted[,"trend"]
     if (gamma > 0)
@@ -196,7 +212,7 @@ function (x,
                    gamma     = gamma,
                    coefficients = c(a = final.fit$level[len + 1],
                                     b = if (beta > 0) final.fit$trend[len + 1],
-                                    s = if (gamma > 0) final.fit$season[len + 1:f]),
+                                    s = if (gamma > 0) final.fit$seasonal[len + 1:f]),
                    seasonal  = seasonal,
                    SSE       = final.fit$SSE,
                    call      = match.call()
@@ -248,7 +264,7 @@ predict.HoltWinters <-
              lwr = if(prediction.interval) fit - int
              ),
        start = end(lag(fitted(object)[,1], k = -1)),
-       freq  = frequency(fitted(object)[,1])
+       frequency  = frequency(fitted(object)[,1])
        )
 }
 
@@ -349,9 +365,9 @@ decompose <- function (x, type = c("additive", "multiplicative"), filter = NULL)
     structure(
               list(seasonal = seasonal,
                    trend    = trend,
-                   random   = if (type == "additive") 
+                   random   = if (type == "additive")
                       x - seasonal - trend
-                   else 
+                   else
                       x / seasonal / trend,
                    figure   = figure,
                    type     = type
