@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -119,9 +119,14 @@ acf(double *x, int *n, int *nser, int *nlag, int *correlation, double *acf)
     if(*correlation) {
 	for(u = 0; u < ns; u++)
 	    se[u] = sqrt(acf[0 + d1*u + d2*u]);
-	for(u = 0; u < ns; u++)
-	    for(v = 0; v < ns; v++)
-		for(lag = 0; lag <= nl; lag++)
-		    acf[lag + d1*u + d2*v] /= se[u]*se[v];
+	if(nn == 1) {
+	    for(u = 0; u < ns; u++)
+	    	acf[0 + d1*u + d2*u] = 1.0;
+	} else {
+	    for(u = 0; u < ns; u++)
+		for(v = 0; v < ns; v++)
+		    for(lag = 0; lag <= nl; lag++)
+			acf[lag + d1*u + d2*v] /= se[u]*se[v];
+	}
     }
 }
