@@ -16,14 +16,15 @@
 
 .onLoad <- function(lib, pkg)
 {
-    packageStartupMessage("Loading Tcl/Tk interface ...", domain = "R-tcltk",
-                          appendLF = FALSE)
+    packageStartupMessage("Loading Tcl/Tk interface ...",
+                          domain = "R-tcltk", appendLF = FALSE)
     if(nzchar(Sys.getenv("MY_TCLTK"))) {
         library.dynam("tcltk", pkg, lib)
     } else {
         if(!file.exists(file.path(R.home(), "Tcl")))
             stop("Tcl/Tk support files were not installed", call.=FALSE)
-        Sys.setenv("TCL_LIBRARY"=file.path(R.home(), "Tcl/lib/tcl8.4"))
+##        Sys.setenv("TCL_LIBRARY" =
+##                   file.path(gsub("\\\\", "/", R.home()), "Tcl/lib/tcl8.4"))
         opath <-  Sys.getenv("PATH")
         tclbin <- file.path(R.home(), "Tcl/bin")
         Sys.setenv(PATH=paste(tclbin, opath, sep=";"))
@@ -31,18 +32,17 @@
         Sys.setenv(PATH=opath)
     }
     .C("tcltk_start", PACKAGE="tcltk")
-    extra <- system.file("exec", package = "tcltk")
-    extra <- gsub("\\\\", "/", extra)
-    addTclPath(extra)
+    addTclPath(system.file("exec", package = "tcltk"))
     packageStartupMessage(" ", "done", domain = "R-tcltk")
     invisible()
 }
 
 .onUnload <- function(libpath) {
-    if(is.loaded(symbol.C("tcltk_end"))) {
+    ## precaution in case the DLL has been unloaded without the namespace
+    if(is.loaded("tcltk_end", PACKAGE="tcltk")) {
         .C("tcltk_end", PACKAGE="tcltk")
-## unloading the DLL used to work, but it seems Tcl/Tk 8.4.1 does
-## not like being reinitialized
+        ## unloading the DLL used to work with 8.3,
+        ## but it seems Tcl/Tk 8.4.x does not like being reinitialized
         ## library.dynam.unload("tcltk", libpath)
     }
 }
