@@ -198,9 +198,9 @@ old.packages <- function(lib.loc = NULL, repos = getOption("repos"),
         lib.loc <- .libPaths()
 
     instp <- installed.packages(lib.loc = lib.loc)
-    if(NROW(instp) == 0)
-        stop(gettextf("no installed packages for (invalid?) 'lib.loc=%s'",
-                      lib.loc), domain = NA)
+    if(NROW(instp) == 0) return(NULL)
+    ##    stop(gettextf("no installed packages for (invalid?) 'lib.loc=%s'",
+    ##                  lib.loc), domain = NA)
     if(is.null(available))
         available <- available.packages(contriburl = contriburl,
                                         method = method)
@@ -551,28 +551,20 @@ contrib.url <- function(repos, type = getOption("pkgType"))
 getCRANmirrors <- function(all=FALSE, local.only=FALSE)
 {
     m <- NULL
-    if(!local.only){
-        m <- try(read.csv(url("http://cran.r-project.org/CRAN_mirrors.csv"),
-                          as.is=TRUE))
-    }
-
-    if(is.null(m) || inherits(m, "try-error")){
-        m <- read.csv(file.path(R.home("doc"), "CRAN_mirrors.csv"),
-                      as.is=TRUE)
-    }
-
-    if(!is.null(m$OK)){
-        m$OK <- as.logical(m$OK)
-
-        if(!all){
-            m <- m[m$OK,]
-        }
+    if(!local.only) {
+        ## try to handle explicitly failure to connect to CRAN.
+        con <- url("http://cran.r-project.org/CRAN_mirrors.csv")
+        m <- try(open(con, "r"), silent = TRUE)
+        if(!inherits(m, "try-error")) m <- try(read.csv(con, as.is=TRUE))
+        close(con)
     }
+    if(is.null(m) || inherits(m, "try-error"))
+        m <- read.csv(file.path(R.home("doc"), "CRAN_mirrors.csv"), as.is=TRUE)
+    if(!all) m <- m[as.logical(m$OK), ]
     m
 }
 
 
-
 chooseCRANmirror <- function(graphics = getOption("menu.graphics"))
 {
     if(!interactive()) stop("cannot choose a CRAN mirror non-interactively")
