@@ -1,3 +1,19 @@
+#  File src/library/utils/R/readNEWS.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 readNEWS <- function(file = file.path(R.home(), "NEWS"),
                      trace = FALSE, chop = c("first", "1", "par1", "keepAll"))
 {
@@ -72,10 +88,9 @@ readNEWS <- function(file = file.path(R.home(), "NEWS"),
 	entries <- as.list(iS)
 	## entries have no labels !
 
-	iS <- c(iS, nl+1:1)
-	for(i in seq(along = entries)) {
-	    entries[[i]] <- parseEntry(ll[iS[i] : (iS[i+1] - 1:1)])
-	}
+	iS <- c(iS, nl+1L)
+	for(i in seq_along(entries))
+	    entries[[i]] <- parseEntry(ll[iS[i] : (iS[i+1] - 1L)])
 	entries
     }
 
@@ -90,11 +105,10 @@ readNEWS <- function(file = file.path(R.home(), "NEWS"),
 	if(trace) cat("	 with ", length(iC), "sections\n")
 	sections <- as.list(iC)
 	names(sections) <- ll[iC]
-	iC <- c(iC, length(ll)+1:1) # such that	 iC[i] : (iC[i+1]-1)  makes sense
-	for(i in seq(along = sections)) {
-	    sections[[i]] <- parseSection(ll[(iC[i]+ 1:1) : (iC[i+1] - 1:1)],
+	iC <- c(iC, length(ll)+1L) # such that	 iC[i] : (iC[i+1]-1)  makes sense
+	for(i in seq_along(sections))
+	    sections[[i]] <- parseSection(ll[(iC[i]+ 1L) : (iC[i+1] - 1L)],
 					  kind = names(sections)[i])
-	}
 	sections
     }
 
@@ -109,11 +123,10 @@ readNEWS <- function(file = file.path(R.home(), "NEWS"),
 	versions <- as.list(iC)
 	names(versions) <- sub(s.pre, '', ll[iC])
 
-	iC <- c(iC, length(ll)+1:1) # such that	 iC[i] : (iC[i+1]-1)  makes sense
-	for(i in seq(along = versions)) {
-	    versions[[i]] <- parseVersion(ll[(iC[i]+ 1:1) : (iC[i+1] - 1:1)],
+	iC <- c(iC, length(ll)+1L) # such that	 iC[i] : (iC[i+1]-1)  makes sense
+	for(i in seq_along(versions))
+	    versions[[i]] <- parseVersion(ll[(iC[i]+ 1L) : (iC[i+1] - 1L)],
 					  ver = names(versions)[i])
-	}
 	versions
     }
 
@@ -153,11 +166,10 @@ readNEWS <- function(file = file.path(R.home(), "NEWS"),
         cat("Now parsing each: ...\n")
     }
 
-    iS <- c(iS, nl+1:1) # such that  iS[i] : (iS[i+1]-1)  makes sense
-    for(i in seq(along = series)) {
-	series[[i]] <- parseSeries(ll[(iS[i]+ 1:1) : (iS[i+1] - 1:1)],
+    iS <- c(iS, nl+1L) # such that  iS[i] : (iS[i+1]-1)  makes sense
+    for(i in seq_along(series))
+	series[[i]] <- parseSeries(ll[(iS[i]+ 1L) : (iS[i+1] - 1L)],
 				   ver = names(series)[i])
-    }
     attr(series, "call") <- cl
     class(series) <- "newsTree"
     series
