@@ -1,3 +1,19 @@
+#  File src/library/utils/R/str.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 ####------ str : show STRucture of an R object
 str <- function(object, ...) UseMethod("str")
 
@@ -12,7 +28,7 @@ str.data.frame <- function(object, ...)
     ## Show further classes // Assume that they do NOT have an own Method --
     ## not quite perfect ! (.Class = 'remaining classes', starting with current)
     cl <- oldClass(object); cl <- cl[cl != "data.frame"]  #- not THIS class
-    if(0 < length(cl)) cat("Classes", cl, " and ")
+    if(0 < length(cl)) cat("Classes", paste(sQuote(cl), collapse=", "), "and ")
 
     cat("'data.frame':	", nrow(object), " obs. of  ",
 	(p <- length(object)), " variable", if(p != 1)"s", if(p > 0)":",
@@ -24,13 +40,44 @@ str.data.frame <- function(object, ...)
     else invisible(NextMethod("str", give.length=FALSE,...))
 }
 
+str.POSIXt <- function(object, ...) {
+    cl <- oldClass(object)
+    ## be careful to be fast for large object:
+    n <- length(object)
+    if(n >= 1000) object <- object[1:1000]
+
+    give.length <- TRUE ## default
+    ## use 'give.length' when specified, else default = give.head
+    if(length(larg <- list(...))) {
+	nl <- names(larg)
+	iGiveHead <- which(nl == "give.head")
+	if (any(Bgl <- nl == "give.length"))
+	    give.length <- larg[[which(Bgl)]]
+	else if(length(iGiveHead))
+	    give.length <- larg[[iGiveHead]]
+	if(length(iGiveHead)) # eliminate it from arg.list
+	    larg <- larg[ - iGiveHead ]
+	if(is.numeric(larg[["nest.lev"]]) &&
+	   is.numeric(v.len <- larg[["vec.len"]])) # typical call from data.frame
+	    ## diminuish length for typical call:
+	    larg[["vec.len"]] <-
+		min(larg[["vec.len"]],
+		    (larg[["width"]]- nchar(larg[["indent.str"]]) -31)%/% 19)
+    }
+
+    le.str <- if(give.length) paste("[1:",as.character(n),"]", sep="")
+    cat(" ", cl[min(2, length(cl))], le.str,", format: ", sep = "")
+    do.call(str, c(list(format(object), give.head = FALSE), larg))
+}
+
 strOptions <- function(strict.width = "no", digits.d = 3, vec.len = 4)
     list(strict.width = strict.width, digits.d = digits.d, vec.len = vec.len)
 
 str.default <-
     function(object, max.level = NA, vec.len = strO$vec.len,
              digits.d = strO$digits.d,
-	     nchar.max = 128, give.attr = TRUE, give.length = TRUE,
+	     nchar.max = 128, give.attr = TRUE,
+             give.head = TRUE, give.length = give.head,
 	     width = getOption("width"), nest.lev = 0,
 	     indent.str= paste(rep.int(" ", max(0,nest.lev+1)), collapse= ".."),
 	     comp.str="$ ", no.list = FALSE, envir = baseenv(),
@@ -57,7 +104,7 @@ str.default <-
 	ss <- capture.output(str(object, max.level = max.level,
 				 vec.len = vec.len, digits.d = digits.d,
 				 nchar.max = nchar.max,
-				 give.attr= give.attr, give.length= give.length,
+				 give.attr= give.attr, give.head= give.head, give.length= give.length,
 				 width = width, nest.lev = nest.lev,
 				 indent.str = indent.str, comp.str= comp.str,
 				 no.list= no.list || is.data.frame(object),
@@ -85,6 +132,9 @@ str.default <-
 	if(any(ii)) x[ii] <- P0(trimmed[ii], Sep, ch)
 	x
     }
+    pClass <- function(cls)
+	paste("Class", if(length(cls) > 1) "es",
+              " '", paste(cls, collapse = "', '"), "' ", sep="")
 
     ## le.str: not used for arrays:
     le.str <-
@@ -103,14 +153,11 @@ str.default <-
     mod <- ""; char.like <- FALSE
     if(give.attr) a <- attributes(object)#-- save for later...
 
-    if(is.function(object)) {
-	cat(if(is.null(ao <- args(object))) deparse(object)
-	else { dp <- deparse(ao); paste(dp[-length(dp)], collapse="\n") },"\n")
-    } else if (is.null(object))
+    if (is.null(object))
 	cat(" NULL\n")
     else if(S4) {
 	a <- sapply(methods::.slotNames(object), methods::slot,
-                    object=object, simplify = FALSE)
+		    object=object, simplify = FALSE)
 	cat("Formal class", " '", paste(cl, collapse = "', '"),
 	    "' [package \"", attr(cl,"package"), "\"] with ",
 	    length(a)," slots\n", sep="")
@@ -120,14 +167,21 @@ str.default <-
 	    nchar.max = nchar.max, give.attr = give.attr, width=width)
 	return(invisible())
     }
-    else if(is.list(object)) {
+    else if(is.function(object)) {
+	cat(if(is.null(ao <- args(object))) deparse(object)
+	else { dp <- deparse(ao); paste(dp[-length(dp)], collapse="\n") },"\n")
+    } else if(is.list(object)) {
 	i.pl <- is.pairlist(object)
 	is.d.f <- is.data.frame(object)
 	##?if(is.d.f) std.attr <- c(std.attr, "class", if(is.d.f) "row.names")
 	if(le == 0) {
 	    if(is.d.f) std.attr <- c(std.attr, "class", "row.names")
 	    else cat(" ", if(i.pl)"pair", "list()\n",sep="")
-	} else {
+	} else { # list, length >= 1 :
+	    if(irregCl <- has.class && identical(object[[1]], object)) {
+		le <- length(object <- unclass(object))
+		std.attr <- c(std.attr, "class")
+	    }
 	    if(no.list || (has.class &&
 			   any(sapply(paste("str", cl, sep="."),
 					#use sys.function(.) ..
@@ -136,7 +190,8 @@ str.default <-
 		## str.default is a 'NextMethod' : omit the 'List of ..'
 		std.attr <- c(std.attr, "class", if(is.d.f) "row.names")
 	    } else {
-		cat(if(i.pl) "Dotted pair list" else "List",
+		cat(if(i.pl) "Dotted pair list" else
+		    if(irregCl) paste(pClass(cl), "hidden list") else "List",
 		    " of ", le, "\n", sep="")
 	    }
 	    if (is.na(max.level) || nest.lev < max.level) {
@@ -155,7 +210,7 @@ str.default <-
                         indent.str = paste(indent.str,".."),
                         nchar.max = nchar.max, max.level = max.level,
                         vec.len = vec.len, digits.d = digits.d,
-                        give.attr = give.attr, give.length = give.length,
+                        give.attr = give.attr, give.head= give.head, give.length = give.length,
                         width = width, envir = envir)
 		}
 	    }
@@ -236,7 +291,7 @@ str.default <-
 	    if(!is.character(lev.att)) {# should not happen..
 		warning("'object' does not have valid levels()")
 		nl <- 0
-	    } else lev.att <- encodeString(lev.att, na = FALSE, quote = '"')
+	    } else lev.att <- encodeString(lev.att, na.encode = FALSE, quote = '"')
 	    ord <- is.ordered(object)
 	    object <- unclass(object)
 	    if(nl) {
@@ -264,12 +319,14 @@ str.default <-
 		  c("externalptr", "weakref", "environment")) {
 	    ## Careful here, we don't want to change pointer objects
 	    if(has.class)
-		cat("Class", if(length(cl) > 1) "es",
-		" '", paste(cl, collapse = "', '"), "' ", sep="")
+                cat(pClass(cl))
 	    le <- v.len <- 0
 	    str1 <- paste("<", typeof(object), ">", sep="")
-	    if(typeof(object) == "environment")
-		str1 <- paste("length", length(object), str1)
+	    if(typeof(object) == "environment") {
+                nm <- environmentName(object)
+                str1 <- if(nchar(nm)[1]) paste("<", nm , ">", sep="")
+		else paste("length", length(object), str1)
+            }
 	    has.class <- TRUE # fake for later
 	    std.attr <- "class"
 	    ## ideally we would figure out if as.character has a
@@ -346,7 +403,7 @@ str.default <-
 	    if(give.mode) str1 <- P0(str1, ', mode "', mod,'":')
 
 	} else if(is.logical(object)) {
-	    v.len <- 3 * v.len
+	    v.len <- 1.5 * v.len # was '3' originally (but S prints 'T' 'F' ..)
 	    format.fun <- format
 	} else if(is.numeric(object)) {
 	    iv.len <- round(2.5 * v.len)
@@ -381,7 +438,7 @@ str.default <-
 	    ## if object is very long, drop the rest which won't be used anyway:
 	    max.len <- max(100, width %/% 3 + 1, if(!missing(vec.len)) vec.len)
 	    if(le > max.len) object <- object[1:max.len]
-	    encObj <- encodeString(object, quote= '"', na= FALSE)
+	    encObj <- encodeString(object, quote= '"', na.encode= FALSE)
 					#O: encodeString(object)
 	    v.len <-
 		if(missing(vec.len)) {
@@ -392,7 +449,7 @@ str.default <-
 	    ile <- min(le, v.len)
 	    if(ile >= 1) ## truncate if LONG char:
 		object <- maybe_truncate(encObj[1:ile])
-					#O: encodeString(object, quote= '"', na= FALSE)
+					#O: encodeString(object, quote= '"', na.encode= FALSE)
 	    formObj <- function(x) paste(as.character(x), collapse=" ")
 	}
 	else {
@@ -404,8 +461,8 @@ str.default <-
 	    formObj <- function(x) paste(format.fun(x), collapse = " ")
 	}
 
-	cat(str1, " ", formObj(if(ile >= 1) object[1:ile] else
-			       if(v.len > 0) object),
+	cat(if(give.head) P0(str1, " "),
+	    formObj(if(ile >= 1) object[1:ile] else if(v.len > 0) object),
 	    if(le > v.len) " ...", "\n", sep="")
 
     } ## else (not function nor list)----------------------------------------
@@ -420,7 +477,7 @@ str.default <-
 		    max.level = max.level, digits.d = digits.d,
 		    nchar.max = nchar.max,
 		    vec.len = if(nam[i] == "source") 1 else vec.len,
-		    give.attr= give.attr, give.length= give.length, width= width)
+		    give.attr= give.attr, give.head= give.head, give.length= give.length, width= width)
 	    }
     }
     invisible()	 ## invisible(object)#-- is SLOOOOW on large objects
