@@ -1,3 +1,19 @@
+#  File src/library/utils/R/unix/help.start.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 help.start <- function (gui = "irrelevant", browser = getOption("browser"),
 			remote = NULL)
 {
@@ -35,11 +51,11 @@ browseURL <- function(url, browser = getOption("browser"))
     shQuote <- function(string)
         paste('"', gsub("\\$", "\\\\$", string), '"', sep="")
 
-    if(!is.character(url) || !(length(url) == 1) || (nchar(url) == 0))
+    if(!is.character(url) || !(length(url) == 1) || !nzchar(url))
         stop("'url' must be a non-empty character string")
     if(!is.character(browser)
        || !(length(browser) == 1)
-       || (nchar(browser) == 0))
+       || !nzchar(browser))
         stop("'browser' must be a non-empty character string")
 
     if (.Platform$GUI == "AQUA" ||
