@@ -1835,8 +1835,7 @@ SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
                     nb = wcstombs(NULL, wc, 0);
                     cbuf = CallocCharBuf(nb);
                     wcstombs(cbuf, wc, nb + 1);
-                    SET_STRING_ELT(y, i, mkChar(cbuf));
-                    markKnown(STRING_ELT(y, i), STRING_ELT(x, i));
+                    SET_STRING_ELT(y, i, markKnown(cbuf, STRING_ELT(x, i)));
                     Free(cbuf);
                 } else {
                     error(_("invalid multibyte string %d"), i+1);
@@ -1856,8 +1855,7 @@ SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
 		strcpy(xi, translateChar(STRING_ELT(x, i)));
 		for(p = xi; *p != '\0'; p++)
 		    *p = ul ? toupper(*p) : tolower(*p);
-		SET_STRING_ELT(y, i, mkChar(xi));
-		markKnown(STRING_ELT(y, i), STRING_ELT(x, i));
+		SET_STRING_ELT(y, i, markKnown(xi, STRING_ELT(x, i)));
 		Free(xi);
 	    }
 	}
@@ -2224,8 +2222,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
                 nb = wcstombs(NULL, wc, 0);
                 cbuf = CallocCharBuf(nb);
                 wcstombs(cbuf, wc, nb + 1);
-                SET_STRING_ELT(y, i, mkChar(cbuf));
-		markKnown(STRING_ELT(y, i), STRING_ELT(x, i));
+                SET_STRING_ELT(y, i, markKnown(cbuf, STRING_ELT(x, i)));
                 Free(cbuf);
             }
         }
@@ -2282,8 +2279,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 		strcpy(cbuf, xi);
 		for(p = (unsigned char *) cbuf; *p != '\0'; p++)
 		    *p = xtable[*p];
-		SET_STRING_ELT(y, i, mkChar(cbuf));
-		markKnown(STRING_ELT(y, i), STRING_ELT(x, i));
+		SET_STRING_ELT(y, i, markKnown(cbuf, STRING_ELT(x, i)));
 		Free(cbuf);
 	    }
 	}
