@@ -13,8 +13,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  *
  *      Interfaces to POSIX date and time functions.
@@ -40,19 +40,6 @@
 # include <config.h>
 #endif
 
-#if defined(HAVE_GLIBC2)
-#include <features.h>
-# ifndef __USE_POSIX
-#  define __USE_POSIX		/* for tzset */
-# endif
-# ifndef __USE_BSD
-#  define __USE_BSD		/* so that we get unsetenv() */
-# endif
-# ifndef __USE_MISC
-#  define __USE_MISC		/* for finite */
-# endif
-#endif
-
 #include <time.h>
 #include <stdlib.h> /* for setenv or putenv */
 #include <Defn.h>
@@ -427,7 +414,7 @@ static struct tm * localtime0(const double *tp, const int local, struct tm *ltm)
 SEXP attribute_hidden do_systime(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans = allocVector(REALSXP, 1);
-#ifdef HAVE_GETTIMEOFDAY
+#if defined(HAVE_GETTIMEOFDAY) && !defined(Win32)
     struct timeval tv;
     int res = gettimeofday(&tv, NULL);
     if(res == 0) {
@@ -464,11 +451,13 @@ SEXP attribute_hidden do_systime(SEXP call, SEXP op, SEXP args, SEXP env)
 
 #ifdef Win32
 #define tzname _tzname
+#elif defined(__CYGWIN__)
+extern __declspec(dllimport) char *tzname[2];
 #else /* Unix */
 extern char *tzname[2];
 #endif
 
-static int set_tz(char *tz, char *oldtz)
+static int set_tz(const char *tz, char *oldtz)
 {
     char *p = NULL;
     int settz = 0;
@@ -549,7 +538,8 @@ SEXP attribute_hidden do_asPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP stz, x, ans, ansnames, klass, tzone;
     int i, n, isgmt = 0, valid, settz = 0;
-    char *tz = NULL, oldtz[20] = "";
+    char oldtz[20] = "";
+    const char *tz = NULL;
 
     checkArity(op, args);
     PROTECT(x = coerceVector(CAR(args), REALSXP));
@@ -592,8 +582,7 @@ SEXP attribute_hidden do_asPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
     SET_STRING_ELT(klass, 1, mkChar("POSIXlt"));
     classgets(ans, klass);
     if (isgmt) {
-	PROTECT(tzone = allocVector(STRSXP, 1));
-	SET_STRING_ELT(tzone, 0, mkChar(tz));
+	PROTECT(tzone = mkString(tz));
     } else {
 	PROTECT(tzone = allocVector(STRSXP, 3));
 	SET_STRING_ELT(tzone, 0, mkChar(tz));
@@ -611,7 +600,8 @@ SEXP attribute_hidden do_asPOSIXct(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP stz, x, ans;
     int i, n = 0, isgmt = 0, nlen[9], settz = 0;
-    char *tz = NULL, oldtz[20] = "";
+    char oldtz[20] = "";
+    const char *tz = NULL;
     struct tm tm;
     double tmp;
 
@@ -681,7 +671,8 @@ SEXP attribute_hidden do_formatPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, sformat, ans, tz;
     int i, n = 0, m, N, nlen[9], UseTZ;
-    char buff[300], *p;
+    char buff[300];
+    const char *p;
     struct tm tm;
 
     checkArity(op, args);
@@ -729,7 +720,8 @@ SEXP attribute_hidden do_formatPOSIXlt(SEXP call, SEXP op, SEXP args, SEXP env)
 	} else {
 	    if(validate_tm(&tm) < 0) SET_STRING_ELT(ans, i, NA_STRING);
 	    else {
-		char *q = CHAR(STRING_ELT(sformat, i%m)), buf2[500];
+		const char *q = CHAR(STRING_ELT(sformat, i%m));
+                char buf2[500];
 		strcpy(buf2,  q);
 		p = strstr(q, "%OS");
 		if(p) {
@@ -819,7 +811,8 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP x, sformat, ans, ansnames, klass, stz, tzone;
     int i, n, m, N, invalid, isgmt = 0, settz = 0;
     struct tm tm, tm2;
-    char *tz = NULL, oldtz[20] = "";
+    const char *tz = NULL;
+    char oldtz[20] = "";
     double psecs = 0.0;
 
     checkArity(op, args);
@@ -886,8 +879,7 @@ SEXP attribute_hidden do_strptime(SEXP call, SEXP op, SEXP args, SEXP env)
     SET_STRING_ELT(klass, 1, mkChar("POSIXlt"));
     classgets(ans, klass);
     if (isgmt) {
-	PROTECT(tzone = allocVector(STRSXP, 1));
-	SET_STRING_ELT(tzone, 0, mkChar(tz));
+	PROTECT(tzone = mkString(tz));
 	setAttrib(ans, install("tzone"), tzone);
 	UNPROTECT(1);
     } else if(strlen(tz)) {
@@ -1006,8 +998,7 @@ SEXP attribute_hidden do_POSIXlt2D(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
     }
 
-    PROTECT(klass = allocVector(STRSXP, 1));
-    SET_STRING_ELT(klass, 0, mkChar("Date"));
+    PROTECT(klass = mkString("Date"));
     classgets(ans, klass);
     UNPROTECT(3);
     return ans;
