@@ -295,10 +295,9 @@ do_pgsub(const char *spat, const char *srep, SEXP vec, int global,
 	    for (j = offset ; s[j] ; j++)
 		*u++ = s[j];
 	    *u = '\0';
-            SET_STRING_ELT(ans, i, mkChar(cbuf));
+            SET_STRING_ELT(ans, i, markKnown(cbuf, STRING_ELT(vec, i)));
             Free(cbuf);
 	}
-	markKnown(STRING_ELT(ans, i), STRING_ELT(vec, i));
     }
     pcre_free(re_pe);
     pcre_free(re_pcre);
