@@ -1399,8 +1399,8 @@ SEXP attribute_hidden do_pathexpand(SEXP call, SEXP op, SEXP args, SEXP rho)
     n = length(fn);
     PROTECT(ans = allocVector(STRSXP, n));
     for (i = 0; i < n; i++) {
-	SEXP tmp = mkChar(R_ExpandFileName(translateChar(STRING_ELT(fn, i))));
-	markKnown(tmp, STRING_ELT(fn, i));
+	SEXP tmp = markKnown(R_ExpandFileName(translateChar(STRING_ELT(fn, i))),
+			     STRING_ELT(fn, i));
 	SET_STRING_ELT(ans, i, tmp);
     }
     UNPROTECT(1);
