@@ -1,7 +1,8 @@
 /*
  *  Mathlib : A C Library of Special Functions
- *  Copyright (C) 2005	Morten Welinder <terra@gnome.org>
- *  Copyright (C) 2005	The R Foundation
+ *  Copyright (C) 2005-6 Morten Welinder <terra@gnome.org>
+ *  Copyright (C) 2005-6 The R Foundation
+ *  Copyright (C) 2006	 The R Core Development Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,10 +14,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  A copy of the GNU General Public License is available via WWW at
- *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
- *  writing to the Free Software Foundation, Inc., 59 Temple Place,
- *  Suite 330, Boston, MA  02111-1307  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  *  SYNOPSIS
  *
@@ -55,22 +55,22 @@
  *	make CFLAGS='-DDEBUG_p -g -I/usr/local/include -I../include'
  */
 
-
 /* Scalefactor:= (2^32)^8 = 2^256 = 1.157921e+77 */
 #define SQR(x) ((x)*(x))
 static const double scalefactor = SQR(SQR(SQR(4294967296.0)));
 #undef SQR
 
-/* If |x| > |k| * M_cutoff,  then  log[ exp(-x) * k^x ]  =~=  -x */
+/* If |x| > |k| * M_cutoff,  then  log[ exp(-x) * k^x ]	 =~=  -x */
 static const double M_cutoff = M_LN2 * DBL_MAX_EXP / DBL_EPSILON;/*=3.196577e18*/
 
 /* Continued fraction for calculation of
- *    1/i + x/(i+d) + x^2/(i+2*d) + x^3/(i+3*d) + ...
+ *    1/i + x/(i+d) + x^2/(i+2*d) + x^3/(i+3*d) + ... = sum_{k=0}^Inf x^k/(i+k*d)
  *
  * auxilary in log1pmx() and lgamma1p()
  */
 static double
-logcf (double x, double i, double d)
+logcf (double x, double i, double d,
+       double eps /* ~ relative tolerance */)
 {
     double c1 = 2 * d;
     double c2 = i + d;
@@ -79,7 +79,6 @@ logcf (double x, double i, double d)
     double b1 = i * (c2 - i * x);
     double b2 = d * d * x;
     double a2 = c4 * c2 - b2;
-    const double cfVSmall = 1.0e-14;/* ~= relative tolerance */
 
 #if 0
     assert (i > 0);
@@ -88,7 +87,7 @@ logcf (double x, double i, double d)
 
     b2 = c4 * b1 - i * b2;
 
-    while (fabs (a2 * b1 - a1 * b2) > fabs (cfVSmall * b1 * b2)) {
+    while (fabs(a2 * b1 - a1 * b2) > fabs(eps * b1 * b2)) {
 	double c3 = c2*c2*x;
 	c2 += d;
 	c4 += d;
@@ -121,18 +120,20 @@ logcf (double x, double i, double d)
 double log1pmx (double x)
 {
     static const double minLog1Value = -0.79149064;
-    static const double two = 2;
 
     if (x > 1 || x < minLog1Value)
 	return log1p(x) - x;
     else { /* expand in	 [x/(2+x)]^2 */
 	double term = x / (2 + x);
 	double y = term * term;
-	if (fabs(x) < 1e-2)
+	if (fabs(x) < 1e-2) {
+	    static const double two = 2;
 	    return term * ((((two / 9 * y + two / 7) * y + two / 5) * y +
 			    two / 3) * y - x);
-	else
-	    return term * (2 * y * logcf (y, 3, 2) - x);
+	} else {
+	    static const double tol_logcf = 1e-14;
+	    return term * (2 * y * logcf (y, 3, 2, tol_logcf) - x);
+	}
     }
 }
 
@@ -142,11 +143,11 @@ double lgamma1p (double a)
 {
     const double eulers_const =	 0.5772156649015328606065120900824024;
 
-    /* coeffs[i] holds (zeta(i+2)-1)/(i+2) , i = 1:N, N = 40 : */
+    /* coeffs[i] holds (zeta(i+2)-1)/(i+2) , i = 0:(N-1), N = 40 : */
     const int N = 40;
     static const double coeffs[40] = {
-	0.3224670334241132182362075833230126e-0,
-	0.6735230105319809513324605383715000e-1,
+	0.3224670334241132182362075833230126e-0,/* = (zeta(2)-1)/2 */
+	0.6735230105319809513324605383715000e-1,/* = (zeta(3)-1)/3 */
 	0.2058080842778454787900092413529198e-1,
 	0.7385551028673985266273097291406834e-2,
 	0.2890510330741523285752988298486755e-2,
@@ -184,10 +185,11 @@ double lgamma1p (double a)
 	0.9573630387838555763782200936508615e-13,
 	0.4664076026428374224576492565974577e-13,
 	0.2273736960065972320633279596737272e-13,
-	0.1109139947083452201658320007192334e-13
+	0.1109139947083452201658320007192334e-13/* = (zeta(40+1)-1)/(40+1) */
     };
 
     const double c = 0.2273736845824652515226821577978691e-12;/* zeta(N+2)-1 */
+    const double tol_logcf = 1e-14;
     double lgam;
     int i;
 
@@ -196,7 +198,7 @@ double lgamma1p (double a)
 
     /* Abramowitz & Stegun 6.1.33,
      * also  http://functions.wolfram.com/06.11.06.0008.01 */
-    lgam = c * logcf (-a / 2, N + 2, 1);
+    lgam = c * logcf(-a / 2, N + 2, 1, tol_logcf);
     for (i = N - 1; i >= 0; i--)
 	lgam = coeffs[i] - a * lgam;
 
@@ -333,16 +335,16 @@ pd_upper_series (double x, double y, int log_p)
     } while (term > sum * DBL_EPSILON);
 
     /* sum =  \sum_{n=1}^ oo  x^n     / (y*(y+1)*...*(y+n-1))
-     *     =  \sum_{n=0}^ oo  x^(n+1) / (y*(y+1)*...*(y+n))
-     *     =  x/y * (1 + \sum_{n=1}^oo  x^n / ((y+1)*...*(y+n)))
-     *     ~  x/y +  o(x/y)   {which happens when alph -> Inf}
+     *	   =  \sum_{n=0}^ oo  x^(n+1) / (y*(y+1)*...*(y+n))
+     *	   =  x/y * (1 + \sum_{n=1}^oo	x^n / ((y+1)*...*(y+n)))
+     *	   ~  x/y +  o(x/y)   {which happens when alph -> Inf}
      */
     return log_p ? log (sum) : sum;
 }
 
 /* Continued fraction for calculation of
  *    ???
- *  =  (i / d)  +  o(i/d)
+ *  =  (i / d)	+  o(i/d)
  */
 static double
 pd_lower_cf (double i, double d)
@@ -399,7 +401,8 @@ pd_lower_cf (double i, double d)
 	}
     }
 
-    REprintf(" ** NON-convergence in pgamma()'s pd_lower_cf() f= %g.\n", f);
+    MATHLIB_WARNING(" ** NON-convergence in pgamma()'s pd_lower_cf() f= %g.\n",
+		    f);
     return f;/* should not happen ... */
 } /* pd_lower_cf() */
 #undef NEEDED_SCALE
@@ -419,8 +422,8 @@ pd_lower_series (double lambda, double y)
 	y--;
     }
     /* sum =  \sum_{n=0}^ oo  y*(y-1)*...*(y - n) / lambda^(n+1)
-     *     =  y/lambda * (1 + \sum_{n=1}^Inf  (y-1)*...*(y-n) / lambda^n
-     *     ~  y/lambda + o(y/lambda)
+     *	   =  y/lambda * (1 + \sum_{n=1}^Inf  (y-1)*...*(y-n) / lambda^n
+     *	   ~  y/lambda + o(y/lambda)
      */
 #ifdef DEBUG_p
     REprintf(" done: term=%g, sum=%g, y= %g\n", term, sum, y);
@@ -436,7 +439,7 @@ pd_lower_series (double lambda, double y)
 	REprintf(" y not int: add another term ");
 #endif
 	/* FIXME: in quite few cases, adding  term*f  has no effect (f too small)
-	 *        and unnecessary e.g. for pgamma(4e12, 121.1) */
+	 *	  and unnecessary e.g. for pgamma(4e12, 121.1) */
 	f = pd_lower_cf (y, lambda + 1 - y);
 #ifdef DEBUG_p
 	REprintf("  (= %.14g) * term = %.14g to sum %g\n", f, term * f, sum);
@@ -448,86 +451,140 @@ pd_lower_series (double lambda, double y)
 } /* pd_lower_series() */
 
 /*
- * Asymptotic expansion to calculate the probability that poisson variate
+ * Compute the following ratio with higher accuracy that would be had
+ * from doing it directly.
+ *
+ *		 dnorm (x, 0, 1, FALSE)
+ *	   ----------------------------------
+ *	   pnorm (x, 0, 1, lower_tail, FALSE)
+ *
+ * Abramowitz & Stegun 26.2.12
+ */
+static double
+dpnorm (double x, int lower_tail, double lp)
+{
+    /*
+     * So as not to repeat a pnorm call, we expect
+     *
+     *	 lp == pnorm (x, 0, 1, lower_tail, TRUE)
+     *
+     * but use it only in the non-critical case where either x is small
+     * or p==exp(lp) is close to 1.
+     */
+
+    if (x < 0) {
+	x = -x;
+	lower_tail = !lower_tail;
+    }
+
+    if (x > 10 && !lower_tail) {
+	double term = 1 / x;
+	double sum = term;
+	double x2 = x * x;
+	double i = 1;
+
+	do {
+	    term *= -i / x2;
+	    sum += term;
+	    i += 2;
+	} while (fabs (term) > DBL_EPSILON * sum);
+
+	return 1 / sum;
+    } else {
+	double d = dnorm (x, 0, 1, FALSE);
+	return d / exp (lp);
+    }
+}
+
+/*
+ * Asymptotic expansion to calculate the probability that Poisson variate
  * has value <= x.
+ * Various assertions about this are made (without proof) at
+ * http://members.aol.com/iandjmsmith/PoissonApprox.htm
  */
 static double
 ppois_asymp (double x, double lambda, int lower_tail, int log_p)
 {
-    static const double coef15 = 1/12.;
-    static const double coef25 = 1/288.;
-    static const double coef35 = -139/51840.;
-    static const double coef45 = -571/2488320.;
-    static const double coef55 = 163879/209018880.;
-    static const double coef65 =  5246819/75246796800.;
-    static const double coef75 = -534703531/902961561600.;
-    static const double coef1 = 2/3.;
-    static const double coef2 = -4/135.;
-    static const double coef3 = 8/2835.;
-    static const double coef4 = 16/8505.;
-    static const double coef5 = -8992/12629925.;
-    static const double coef6 = -334144/492567075.;
-    static const double coef7 = 698752/1477701225.;
-    static const double two = 2;
+    static const double coefs_a[8] = {
+	-1e99, /* placeholder used for 1-indexing */
+	2/3.,
+	-4/135.,
+	8/2835.,
+	16/8505.,
+	-8992/12629925.,
+	-334144/492567075.,
+	698752/1477701225.
+    };
 
-    double dfm, pt_,s2pt,res1,res2,elfb,term;
-    double ig2,ig3,ig4,ig5,ig6,ig7,ig25,ig35,ig45,ig55,ig65,ig75;
-    double f, np, nd;
+    static const double coefs_b[8] = {
+	-1e99, /* placeholder */
+	1/12.,
+	1/288.,
+	-139/51840.,
+	-571/2488320.,
+	163879/209018880.,
+	5246819/75246796800.,
+	-534703531/902961561600.
+    };
+
+    double elfb, elfb_term;
+    double res12, res1_term, res1_ig, res2_term, res2_ig;
+    double dfm, pt_, s2pt, f, np;
+    int i;
 
     dfm = lambda - x;
-    pt_ = -x * log1pmx (dfm / x);
-    s2pt = sqrt (2 * pt_);
+    /* If lambda is large, the distribution is highly concentrated
+       about lambda.  So representation error in x or lambda can lead
+       to arbitrarily large values of pt_ and hence divergence of the
+       coefficients of this approximation.
+    */
+    pt_ = - log1pmx (dfm / x);
+    s2pt = sqrt (2 * x * pt_);
     if (dfm < 0) s2pt = -s2pt;
 
-    ig2 = 1.0 + pt_;
-    term = pt_ * pt_ * 0.5;
-    ig3 = ig2 + term;
-    term *= pt_ / 3;
-    ig4 = ig3 + term;
-    term *= pt_ / 4;
-    ig5 = ig4 + term;
-    term *= pt_ / 5;
-    ig6 = ig5 + term;
-    term *= pt_ / 6;
-    ig7 = ig6 + term;
-
-    term = pt_ * (two / 3);
-    ig25 = 1.0 + term;
-    term *= pt_ * (two / 5);
-    ig35 = ig25 + term;
-    term *= pt_ * (two / 7);
-    ig45 = ig35 + term;
-    term *= pt_ * (two / 9);
-    ig55 = ig45 + term;
-    term *= pt_ * (two / 11);
-    ig65 = ig55 + term;
-    term *= pt_ * (two / 13);
-    ig75 = ig65 + term;
-
-    elfb = ((((((coef75/x + coef65)/x + coef55)/x + coef45)/x + coef35)/x +
-	     coef25)/x + coef15) + x;
-    res1 = ((((((ig7*coef7/x + ig6*coef6)/x + ig5*coef5)/x + ig4*coef4)/x +
-	      ig3*coef3)/x + ig2*coef2)/x + coef1)*sqrt(x);
-    res2 = ((((((ig75*coef75/x + ig65*coef65)/x + ig55*coef55)/x + ig45*coef45)/
-	      x + ig35*coef35)/x + ig25*coef25)/x + coef15)*s2pt;
+    res12 = 0;
+    res1_ig = res1_term = sqrt (x);
+    res2_ig = res2_term = s2pt;
+    for (i = 1; i < 8; i++) {
+	res12 += res1_ig * coefs_a[i];
+	res12 += res2_ig * coefs_b[i];
+	res1_term *= pt_ / i ;
+	res2_term *= 2 * pt_ / (2 * i + 1);
+	res1_ig = res1_ig / x + res1_term;
+	res2_ig = res2_ig / x + res2_term;
+    }
 
+    elfb = x;
+    elfb_term = 1;
+    for (i = 1; i < 8; i++) {
+	elfb += elfb_term * coefs_b[i];
+	elfb_term /= x;
+    }
     if (!lower_tail) elfb = -elfb;
-    f = (res1 + res2) / elfb;
-
-    np = pnorm (s2pt, 0.0, 1.0, !lower_tail, log_p);
-    nd = dnorm (s2pt, 0.0, 1.0, log_p);
-
 #ifdef DEBUG_p
-    REprintf ("pp*_asymp(): f=%.14g np=%.14g nd=%.14g  f*nd=%.14g\n",
-	      f, np, nd, f * nd);
+    REprintf ("res12 = %.14g   elfb=%.14g\n", elfb, res12);
 #endif
 
-    if (log_p)
-	return (f >= 0)
-	    ? logspace_add (np, log (fabs (f)) + nd)
-	    : logspace_sub (np, log (fabs (f)) + nd);
-    else
+    f = res12 / elfb;
+
+    np = pnorm (s2pt, 0.0, 1.0, !lower_tail, log_p);
+
+    if (log_p) {
+	double n_d_over_p = dpnorm (s2pt, !lower_tail, np);
+#ifdef DEBUG_p
+	REprintf ("pp*_asymp(): f=%.14g	 np=e^%.14g  nd/np=%.14g  f*nd/np=%.14g\n",
+		  f, np, n_d_over_p, f * n_d_over_p);
+#endif
+	return np + log1p (f * n_d_over_p);
+    } else {
+	double nd = dnorm (s2pt, 0., 1., log_p);
+
+#ifdef DEBUG_p
+	REprintf ("pp*_asymp(): f=%.14g	 np=%.14g  nd=%.14g  f*nd=%.14g\n",
+		  f, np, nd, f * nd);
+#endif
 	return np + f * nd;
+    }
 } /* ppois_asymp() */
 
 
@@ -545,7 +602,8 @@ double pgamma_raw (double x, double alph, int lower_tail, int log_p)
 
     if (x < 1) {
 	res = pgamma_smallx (x, alph, lower_tail, log_p);
-    } else if (x <= alph - 1 && x < 0.8 * (alph + 50)) {/* incl. large alph */
+    } else if (x <= alph - 1 && x < 0.8 * (alph + 50)) {
+	/* incl. large alph compared to x */
 	double sum = pd_upper_series (x, alph, log_p);/* = x/alph + o(x/alph) */
 	double d = dpois_wrap (alph, x, log_p);
 #ifdef DEBUG_p
@@ -558,7 +616,8 @@ double pgamma_raw (double x, double alph, int lower_tail, int log_p)
 		: 1 - d * sum;
 	else
 	    res = log_p ? sum + d : sum * d;
-    } else if (alph - 1 < x && alph < 0.8 * (x + 50)) {/* incl. large x */
+    } else if (alph - 1 < x && alph < 0.8 * (x + 50)) {
+	/* incl. large x compared to alph */
 	double sum;
 	double d = dpois_wrap (alph, x, log_p);
 #ifdef DEBUG_p
@@ -585,7 +644,7 @@ double pgamma_raw (double x, double alph, int lower_tail, int log_p)
 	    res = log_p
 		? R_Log1_Exp (d + sum)
 		: 1 - d * sum;
-    } else {
+    } else { /* x > 1 and x fairly near alph. */
 #ifdef DEBUG_p
 	REprintf(" using ppois_asymp()\n");
 #endif
@@ -646,7 +705,7 @@ double pgamma(double x, double alph, double scale, int lower_tail, int log_p)
 /*
  *  Copyright (C) 1998		Ross Ihaka
  *  Copyright (C) 1999-2000	The R Development Core Team
- *  Copyright (C) 2003-2004     The R Foundation
+ *  Copyright (C) 2003-2004	The R Foundation
  *  based on AS 239 (C) 1988 Royal Statistical Society
  *
  *  ................
@@ -666,7 +725,7 @@ double pgamma(double x, double alph, double scale, int lower_tail, int log_p)
  */
 
 /* now would need this here: */
-double pgamma_raw(x, alph, lower_tail, log_p) {
+double attribute_hidden pgamma_raw(x, alph, lower_tail, log_p) {
     return pgamma(x, alph, 1, lower_tail, log_p);
 }
 
@@ -791,7 +850,7 @@ double pgamma(double x, double alph, double scale, int lower_tail, int log_p)
     if (log_p && lower_tail)
 	return(arg);
     /* else */
-    /* sum = exp(arg); and return   if(lower_tail) sum  else 1-sum : */
+    /* sum = exp(arg); and return   if(lower_tail) sum	else 1-sum : */
     return (lower_tail) ? exp(arg) : (log_p ? R_Log1_Exp(arg) : -expm1(arg));
 }
 
