@@ -15,8 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8> char here is handled as a whole string */
@@ -138,7 +138,6 @@ int Rf_initialize_R(int ac, char **av)
     char *p, msg[1024], cmdlines[10000], **avv;
     structRstart rstart;
     Rstart Rp = &rstart;
-    cmdlines[0] = '\0';
 
 #ifdef ENABLE_NLS
     char localedir[PATH_MAX+20];
@@ -233,6 +232,7 @@ int Rf_initialize_R(int ac, char **av)
        by the R option handler. 
      */
     R_set_command_line_arguments(ac, av);
+    cmdlines[0] = '\0';
 
     /* first task is to select the GUI */
     for(i = 0, avv = av; i < ac; i++, avv++) {
@@ -353,9 +353,11 @@ int Rf_initialize_R(int ac, char **av)
     }
 
     if(strlen(cmdlines)) { /* had at least one -e option */
+	size_t res;
 	if(ifp) R_Suicide(_("cannot use -e with -f or --file"));
 	ifp = tmpfile();
-	fwrite(cmdlines, strlen(cmdlines)+1, 1, ifp);
+	res = fwrite(cmdlines, strlen(cmdlines)+1, 1, ifp);
+	/* FIXME: do something with res */
 	fflush(ifp);
 	rewind(ifp);
     }
