@@ -15,3 +15,8 @@ vector("list", 2^30+2)
 }
 
 getenv("USER") # should produce correct error message.
+
+## bad infinite recursion / on.exit / ... interactions
+bar <- function() 1+1
+foo <- function() { on.exit(bar()); foo() }
+foo() # now simple "infinite recursion"
